@@ -70,9 +70,6 @@ static int transponder_callback(rfid_t* rfid, int event, const rfid_transponder_
   char* id = 0;
   char* event_string = 0;
 
-  if(rfid_cgi->max_count > 0)
-    rfid_cgi->max_count--;
-
   if(rfid_cgi->verbose) {
     int i;
     fprintf(cgiOut, "# ");
@@ -108,15 +105,16 @@ static int transponder_callback(rfid_t* rfid, int event, const rfid_transponder_
 
   free(id);
 
-  if(rfid_cgi->max_count >= 0)
-    return rfid_cgi->max_count == 0 ? RFID_CALLBACK_END : RFID_CALLBACK_OK;
-  else
-    return RFID_CALLBACK_OK;
+  return RFID_CALLBACK_OK;
 }
 
 static int inventory_callback(rfid_t* rfid, rfid_transponder_t** transponders, int transponders_length, void* data)
 {
   int i;
+  rfid_cgi_t* rfid_cgi = (rfid_cgi_t*)data;
+
+  if(rfid_cgi->max_count > 0)
+    rfid_cgi->max_count--;
 
   fprintf(cgiOut, "I");
 
@@ -131,7 +129,10 @@ static int inventory_callback(rfid_t* rfid, rfid_transponder_t** transponders, i
 
   fflush(cgiOut);
 
-  return RFID_CALLBACK_OK;
+  if(rfid_cgi->max_count >= 0)
+    return rfid_cgi->max_count == 0 ? RFID_CALLBACK_END : RFID_CALLBACK_OK;
+  else
+    return RFID_CALLBACK_OK;
 }
 
 /*
@@ -191,14 +192,16 @@ static void usage(const char* message)
   char* usage = "\
 # usage:\n\
 #\n\
-# rfid-cgi?[help=<n>&][device=<special>&][iso=1&][verbose=1&][read=<n>]\n\
-# rfid-cgi?[help=<n>&][device=<special>&][iso=1&[verbose=1&][write=<data>]\n\
+# rfid-cgi?[help=<n>&][device=<special>&][iso=1&][verbose=1&]read=<n>\n\
+# rfid-cgi?[help=<n>&][device=<special>&][iso=1&][verbose=1&][id=<hex>&]write=<data>\n\
+# rfid-cgi?kill=1\n\
 #    read=<n>           Wait for a transponder and display its content.\n\
 #                       Loop <n> times.\n\
 #    write=<data>       Wait for a transponder and store the data\n\
 #                       given in argument in its memory.\n\
 #    id=<hex>           II.II... RFID transponder identifier.\n\
 #    iso=1              Expect ISO-15693 RFID transponders.\n\
+#    kill=1             Kill all running rfid-cgi.\n\
 #    device=<special>   Dialog with RFID reader connected to special\n\
 #                       file <special>. Use %2F instead of /\n\
 #                       (default %2Fdev%2FttyS0).\n\
@@ -214,6 +217,7 @@ static void usage(const char* message)
 #    \n\
 #    [DN]         is one letter describing the event: \n\
 #                 N means the RFID transponder entered the field of the reader,\n\
+#                 D means the RFID transponder left the field of the reader.\n\
 #    II.II...     is a transponder identification in dotted hexadecimal.\n\
 #    BBBB         is a is the size of the memory of the transponder as\n\
 #                 4 digit decimal number\n\
@@ -221,6 +225,11 @@ static void usage(const char* message)
 #                 until a newline is found.\n\
 #    <tab>        is a tabulation used as a separator.\n\
 #\n\
+#    At the end of each read=<n> loop iteration, an inventory of tags\n\
+#    currently in the range of the RFID reader is printed as follows:\n\
+#\n\
+#    I<tab>II.II...<tab>II.II...<tab>II.II... ...\n\
+#\n\
 ";
   fprintf(stderr, "usage: %s\n", message);
   fprintf(cgiOut, "# %s\n%s", message, usage);
@@ -231,6 +240,7 @@ static void usage(const char* message)
 int cgiMain() {
   int retval = 0;
   int option_read = 0;
+  int option_kill = 0;
   int option_write = 0;
   int option_iso = 0;
   char data[RFID_TRANSPONDER_DATA_SIZE + 1];
@@ -269,6 +279,7 @@ int cgiMain() {
   }
 
   cgiFormInteger("iso", &option_iso, 0);
+  cgiFormInteger("kill", &option_kill, 0);
 
   switch(cgiFormString("write", data, RFID_TRANSPONDER_DATA_SIZE)) {
   case cgiFormSuccess:
@@ -285,6 +296,16 @@ int cgiMain() {
     break;
   }
 
+  if(!option_read && !option_write && !option_kill) {
+    usage("Specify at least one of read=<n> write=<data> or kill=1");
+    return -1;
+  }
+
+  if(option_kill) {
+    system("killall rfid-cgi");
+    return 0;
+  }
+
   if(option_read && option_write) {
     usage("Options read=<n> and write=<data> are mutually exclusive");
     return -1;
